@@ -58,7 +58,6 @@ struct _EphyEmbed {
 
   EphyFindToolbar *find_toolbar;
   GtkBox *top_widgets_vbox;
-  GtkPaned *paned;
   WebKitWebView *web_view;
   GSList *destroy_on_transition_list;
   GtkWidget *overlay;
@@ -687,12 +686,31 @@ ephy_embed_mapped_cb (GtkWidget *widget,
   ephy_embed_maybe_load_delayed_request ((EphyEmbed *)widget);
 }
 
+static gboolean
+on_enter_notify_event (GtkWidget        *widget,
+                       GdkEventCrossing *event,
+                       gpointer          user_data)
+{
+  EphyEmbed *embed = EPHY_EMBED (user_data);
+
+  if (event->window != gtk_widget_get_window (embed->floating_bar))
+    return GDK_EVENT_PROPAGATE;
+
+  if (gtk_widget_get_halign (embed->floating_bar) == GTK_ALIGN_START)
+    gtk_widget_set_halign (embed->floating_bar, GTK_ALIGN_END);
+  else
+    gtk_widget_set_halign (embed->floating_bar, GTK_ALIGN_START);
+
+  gtk_widget_queue_allocate (embed->overlay);
+
+  return GDK_EVENT_PROPAGATE;
+}
+
 static void
 ephy_embed_constructed (GObject *object)
 {
   EphyEmbed *embed = (EphyEmbed *)object;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
-  GtkWidget *paned;
   WebKitWebInspector *inspector;
 
   g_signal_connect (shell, "window-restored",
@@ -723,9 +741,9 @@ ephy_embed_constructed (GObject *object)
   gtk_widget_set_halign (embed->floating_bar, GTK_ALIGN_START);
   gtk_widget_set_valign (embed->floating_bar, GTK_ALIGN_END);
   gtk_widget_set_no_show_all (embed->floating_bar, TRUE);
+  g_signal_connect_object (embed->overlay, "enter-notify-event", G_CALLBACK (on_enter_notify_event), embed, 0);
 
   gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), embed->floating_bar);
-  gtk_overlay_set_overlay_pass_through (GTK_OVERLAY (embed->overlay), embed->floating_bar, TRUE);
 
   if (embed->progress_bar_enabled) {
     embed->progress = gtk_progress_bar_new ();
@@ -745,23 +763,18 @@ ephy_embed_constructed (GObject *object)
                       GTK_WIDGET (embed->find_toolbar),
                       FALSE, FALSE, 0);
 
-  paned = GTK_WIDGET (embed->paned);
-
   if (embed->progress_bar_enabled)
     embed->progress_update_handler_id = g_signal_connect (embed->web_view, "notify::estimated-load-progress",
                                                           G_CALLBACK (progress_update), object);
 
-  gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (embed->overlay),
-                   TRUE, FALSE);
-
   gtk_box_pack_start (GTK_BOX (embed),
                       GTK_WIDGET (embed->top_widgets_vbox),
                       FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (embed), paned, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (embed), embed->overlay, TRUE, TRUE, 0);
 
   gtk_widget_show (GTK_WIDGET (embed->top_widgets_vbox));
   gtk_widget_show (GTK_WIDGET (embed->web_view));
-  gtk_widget_show_all (paned);
+  gtk_widget_show_all (embed->overlay);
 
   g_object_connect (embed->web_view,
                     "signal::notify::title", G_CALLBACK (web_view_title_changed_cb), embed,
@@ -806,7 +819,6 @@ ephy_embed_init (EphyEmbed *embed)
   gtk_orientable_set_orientation (GTK_ORIENTABLE (embed),
                                   GTK_ORIENTATION_VERTICAL);
 
-  embed->paned = GTK_PANED (gtk_paned_new (GTK_ORIENTATION_VERTICAL));
   embed->top_widgets_vbox = GTK_BOX (gtk_box_new (GTK_ORIENTATION_VERTICAL, 0));
   embed->seq_context_id = 1;
   embed->seq_message_id = 1;
