@@ -23,7 +23,6 @@
 #include "ephy-find-toolbar.h"
 
 #include "ephy-debug.h"
-#include "ephy-web-view.h"
 #include "contrib/gd-tagged-entry.h"
 
 #include <math.h>
@@ -37,6 +36,7 @@
 struct _EphyFindToolbar {
   HdySearchBar parent_instance;
 
+  GCancellable *cancellable;
   WebKitWebView *web_view;
   WebKitFindController *controller;
   GdTaggedEntry *entry;
@@ -370,23 +370,23 @@ ephy_find_toolbar_load_changed_cb (WebKitWebView   *web_view,
 static void
 ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 {
-  GtkWidget *column;
+  GtkWidget *clamp;
   GtkWidget *box;
   GtkSizeGroup *size_group;
 
   size_group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
 
-  column = GTK_WIDGET (hdy_column_new ());
-  hdy_column_set_maximum_width (HDY_COLUMN (column), 400);
-  hdy_column_set_linear_growth_width (HDY_COLUMN (column), 300);
-  gtk_container_add (GTK_CONTAINER (toolbar), column);
+  clamp = GTK_WIDGET (hdy_clamp_new ());
+  hdy_clamp_set_maximum_size (HDY_CLAMP (clamp), 400);
+  hdy_clamp_set_tightening_threshold (HDY_CLAMP (clamp), 300);
+  gtk_container_add (GTK_CONTAINER (toolbar), clamp);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_style_context_add_class (gtk_widget_get_style_context (box),
                                GTK_STYLE_CLASS_RAISED);
   gtk_style_context_add_class (gtk_widget_get_style_context (box),
                                GTK_STYLE_CLASS_LINKED);
-  gtk_container_add (GTK_CONTAINER (column), box);
+  gtk_container_add (GTK_CONTAINER (clamp), box);
 
   toolbar->entry = gd_tagged_entry_new ();
   toolbar->entry_tag = gd_tagged_entry_tag_new ("");
@@ -436,6 +436,8 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 
   search_entry_changed_cb (GTK_ENTRY (toolbar->entry), toolbar);
 
+  toolbar->cancellable = g_cancellable_new ();
+
   gtk_widget_show_all (GTK_WIDGET (toolbar));
 }
 
@@ -454,6 +456,11 @@ ephy_find_toolbar_dispose (GObject *object)
     toolbar->find_source_id = 0;
   }
 
+  if (toolbar->cancellable) {
+    g_cancellable_cancel (toolbar->cancellable);
+    g_clear_object (&toolbar->cancellable);
+  }
+
   G_OBJECT_CLASS (ephy_find_toolbar_parent_class)->dispose (object);
 }
 
@@ -614,6 +621,39 @@ ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar)
   webkit_find_controller_search_previous (toolbar->controller);
 }
 
+static void
+ephy_find_toolbar_selection_async (GObject      *source_object,
+                                   GAsyncResult *res,
+                                   gpointer      user_data)
+{
+  WebKitWebView *web_view = WEBKIT_WEB_VIEW (source_object);
+  EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (user_data);
+  g_autoptr (GError) error = NULL;
+  g_autoptr (WebKitJavascriptResult) js_result = NULL;
+  JSCValue *value = NULL;
+
+  js_result = webkit_web_view_run_javascript_finish (web_view, res, &error);
+  if (!js_result) {
+    g_warning ("Error running javascript: %s", error->message);
+    return;
+  }
+
+  value = webkit_javascript_result_get_js_value (js_result);
+  if (jsc_value_is_string (value)) {
+    JSCException *exception;
+    g_autofree gchar *str_value = NULL;
+
+    str_value = jsc_value_to_string (value);
+    exception = jsc_context_get_exception (jsc_value_get_context (value));
+    if (exception) {
+      g_warning ("Error running javascript: %s", jsc_exception_get_message (exception));
+    } else if (strlen (str_value)) {
+      gtk_entry_set_text (GTK_ENTRY (toolbar->entry), str_value);
+      gtk_editable_select_region (GTK_EDITABLE (toolbar->entry), 0, -1);
+    }
+  }
+}
+
 void
 ephy_find_toolbar_open (EphyFindToolbar *toolbar,
                         gboolean         links_only,
@@ -624,6 +664,8 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
   toolbar->typing_ahead = typing_ahead;
   toolbar->links_only = links_only;
 
+  webkit_web_view_run_javascript (toolbar->web_view, "window.getSelection().toString();", toolbar->cancellable, ephy_find_toolbar_selection_async, toolbar);
+
   gtk_editable_select_region (GTK_EDITABLE (toolbar->entry), 0, -1);
 
   hdy_search_bar_set_search_mode (HDY_SEARCH_BAR (toolbar), TRUE);
