@@ -20,6 +20,7 @@
 
 #include "config.h"
 #include "ephy-web-process-extension.h"
+#include "ephy-webextension-api.h"
 
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
@@ -56,27 +57,15 @@ struct _EphyWebProcessExtension {
   gboolean is_private_profile;
 
   GHashTable *frames_map;
+  GHashTable *translation_table;
 };
 
 G_DEFINE_TYPE (EphyWebProcessExtension, ephy_web_process_extension, G_TYPE_OBJECT)
 
-static gboolean
-web_page_send_request (WebKitWebPage           *web_page,
-                       WebKitURIRequest        *request,
-                       WebKitURIResponse       *redirected_response,
-                       EphyWebProcessExtension *extension)
+GHashTable *
+ephy_web_process_extension_get_translations (EphyWebProcessExtension *extension)
 {
-  /* FIXME: We should probably remove ephy_remove_tracking_from_uri and instead
-   * trust Intelligent Tracking Prevention to mitigate potential privacy impact
-   * of tracking query parameters. But first we need to enable ITP.
-   */
-  const char *request_uri = webkit_uri_request_get_uri (request);
-  g_autofree char *modified_uri = ephy_remove_tracking_from_uri (request_uri);
-  if (modified_uri && g_strcmp0 (request_uri, modified_uri) != 0) {
-    LOG ("Rewrote %s to %s", request_uri, modified_uri);
-    webkit_uri_request_set_uri (request, modified_uri);
-  }
-  return FALSE;
+  return extension->translation_table;
 }
 
 static void
@@ -119,13 +108,13 @@ web_page_will_submit_form (WebKitWebPage            *web_page,
 }
 
 static char *
-password_form_message_serializer (guint64  frame_id,
+password_form_message_serializer (guint64  page_id,
                                   gboolean is_insecure_action)
 {
   GVariant *variant;
   char *message;
 
-  variant = g_variant_new ("(tb)", frame_id, is_insecure_action);
+  variant = g_variant_new ("(tb)", page_id, is_insecure_action);
   message = g_variant_print (variant, FALSE);
   g_variant_unref (variant);
 
@@ -246,9 +235,6 @@ ephy_web_process_extension_page_created_cb (EphyWebProcessExtension *extension,
   js_context = webkit_frame_get_js_context_for_script_world (webkit_web_page_get_main_frame (web_page), extension->script_world);
   (void)js_context;
 
-  g_signal_connect (web_page, "send-request",
-                    G_CALLBACK (web_page_send_request),
-                    extension);
   g_signal_connect (web_page, "context-menu",
                     G_CALLBACK (web_page_context_menu),
                     extension);
@@ -348,6 +334,18 @@ ephy_web_process_extension_user_message_received_cb (EphyWebProcessExtension *ex
       return;
 
     g_variant_get (parameters, "b", &extension->should_remember_passwords);
+  } else if (g_strcmp0 (name, "WebExtension.Add") == 0) {
+    GVariant *parameters;
+    const char *name;
+    const char *data;
+    guint64 length;
+
+    parameters = webkit_user_message_get_parameters (message);
+    if (!parameters)
+      return;
+
+    g_variant_get (parameters, "(&s&st)", &name, &data, &length);
+    webextensions_add_translation (extension, name, data, length);
   }
 }
 
@@ -398,6 +396,8 @@ ephy_web_process_extension_dispose (GObject *object)
     g_clear_pointer (&extension->frames_map, g_hash_table_unref);
   }
 
+  g_clear_pointer (&extension->translation_table, g_hash_table_destroy);
+
   G_OBJECT_CLASS (ephy_web_process_extension_parent_class)->dispose (object);
 }
 
@@ -667,6 +667,28 @@ window_object_cleared_cb (WebKitScriptWorld       *world,
   js_context = webkit_frame_get_js_context_for_script_world (frame, world);
   jsc_context_push_exception_handler (js_context, (JSCExceptionHandler)js_exception_handler, NULL, NULL);
 
+  /* If we are using the default script world, then we are a WebExtension. We
+   * must not register any internal Epiphany APIs, since they must never be
+   * accessible in the default script world. We don't want them exposed to the
+   * web or to WebExtensions. If we were to improperly allow access to our
+   * internal APIs, then malicious web content could do nasty things like
+   * iterate through passwords stored in EphyPasswordsManager, for example.
+   *
+   * And if we are not using the default script world, then we are not a
+   * WebExtension. There is no point in registering WebExtension APIs, because
+   * WebExtensions only have access to what is in the default script world
+   * anyway.
+   *
+   * FIXME: let's try to make this less confusing:
+   *
+   * https://gitlab.gnome.org/GNOME/epiphany/-/issues/1448
+   * https://gitlab.gnome.org/GNOME/epiphany/-/issues/1449
+   */
+  if (extension->script_world == webkit_script_world_get_default ()) {
+    set_up_webextensions (extension, page, js_context);
+    return;
+  }
+
   bytes = g_resources_lookup_data ("/org/gnome/epiphany-web-process-extension/js/ephy.js", G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
   data = g_bytes_get_data (bytes, &data_size);
   result = jsc_context_evaluate_with_source_uri (js_context, data, data_size, "resource:///org/gnome/epiphany-web-process-extension/js/ephy.js", 1);
@@ -793,7 +815,12 @@ ephy_web_process_extension_initialize (EphyWebProcessExtension *extension,
 
   extension->initialized = TRUE;
 
-  extension->script_world = webkit_script_world_new_with_name (guid);
+  /* Note: An empty guid is used ONLY for WebExtensions which do have an own initialization function */
+  if (strlen (guid) > 0)
+    extension->script_world = webkit_script_world_new_with_name (guid);
+  else
+    extension->script_world = webkit_script_world_get_default ();
+
   g_signal_connect (extension->script_world,
                     "window-object-cleared",
                     G_CALLBACK (window_object_cleared_cb),
@@ -815,4 +842,6 @@ ephy_web_process_extension_initialize (EphyWebProcessExtension *extension,
 
   extension->frames_map = g_hash_table_new_full (g_int64_hash, g_int64_equal,
                                                  g_free, NULL);
+
+  extension->translation_table = g_hash_table_new (g_str_hash, NULL);
 }
