@@ -34,20 +34,31 @@
 #include <string.h>
 #include <fcntl.h>
 
-/* Web Apps are installed in the default config dir of the user.
- * Every app has its own profile directory. To create a web app
- * an id needs to be generated using the given name as
- * <normalized-name>-checksum.
+/* Web apps are installed in the default data dir of the user. Every
+ * app has its own profile directory. To create a web app, an ID needs
+ * to be generated using the given name as <normalized-name>-checksum.
+ * The ID is used to uniquely identify the app.
  *
- * The id is used to uniquely identify the app.
- *  - Program name: org.gnome.Epiphany.WebApp-<id>
- *  - Profile directory: <program-name>
- *  - Desktop file: <profile-dir>/<program-name>.desktop
+ *  - Name: the user-visible pretty app name
+ *  - Normalized name: lowercase name
+ *  - Checksum: SHA-1 of name
+ *  - ID: <normalized-name>-<checksum>
+ *  - GApplication ID: see below
+ *  - Profile directory: <gapplication-id>
+ *  - Desktop file: <profile-dir>/<gapplication-id>.desktop
+ *
+ * Note that our ID and GApplication ID are different. Yes, this is confusing.
+ *
+ * For GApplication ID, there are two cases:
+ *
+ *  - If <id> is safe to use in a D-Bus identifier,
+ *    then: GApplication ID is org.gnome.Epiphany.WebApp-<id>
+ *  - otherwise: GAppliation ID is org.gnome.Epiphany.WebApp-<checksum>
  *
  * System web applications have a profile dir without a desktop file.
  */
 
-#define EPHY_WEB_APP_PROGRAM_NAME_PREFIX "org.gnome.Epiphany.WebApp-"
+#define EPHY_WEB_APP_GAPPLICATION_ID_PREFIX "org.gnome.Epiphany.WebApp-"
 
 char *
 ephy_web_application_get_app_id_from_name (const char *name)
@@ -85,34 +96,97 @@ get_encoded_path (const char *path)
   return encoded;
 }
 
+static const char *
+get_app_id_from_gapplication_id (const char *name)
+{
+  if (!g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
+    g_warning ("GApplication ID %s does not begin with required prefix %s", name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
+    return NULL;
+  }
+
+  return name + strlen (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
+}
+
+static char *
+get_gapplication_id_from_id (const char *id)
+{
+  g_autofree char *gapplication_id = NULL;
+  const char *final_hyphen;
+  const char *checksum;
+
+  /* Ideally we would convert hyphens to underscores here, because
+   * hyphens are not very friendly to D-Bus. However, changing this
+   * would require a new profile migration, because the GApplication ID
+   * must exactly match the desktop file basename.
+   *
+   * If we're willing to do another migration in the future, then we
+   * should drop this path and always return just the prefix plus hash,
+   * using underscores rather than hyphens.
+   */
+  gapplication_id = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, id, NULL);
+  if (g_application_id_is_valid (gapplication_id))
+    return g_steal_pointer (&gapplication_id);
+
+  /* Split ID into: <normalized-name>-<checksum> */
+  final_hyphen = strrchr (id, '-');
+  if (!final_hyphen) {
+    g_warning ("Web app ID %s is broken: must contain a hyphen", id);
+    return NULL;
+  }
+  checksum = final_hyphen + 1;
+
+  if (*checksum == '\0') {
+    g_warning ("Web app ID %s is broken: should end with checksum, not hyphen", id);
+    return NULL;
+  }
+
+  /* We'll simply omit the <normalized-name> from the app ID, in order
+   * to avoid problematic characters. Ideally we would use an underscore
+   * here too, rather than a hyphen, but let's be consistent with
+   * existing web apps.
+   */
+  g_clear_pointer (&gapplication_id, g_free);
+  gapplication_id = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, checksum, NULL);
+
+  if (!g_application_id_is_valid (gapplication_id)) {
+    g_warning ("Web app ID %s is broken: derived GApplication ID %s is not a valid app ID (is the final component alphanumeric?)", id, gapplication_id);
+    return NULL;
+  }
+
+  return g_steal_pointer (&gapplication_id);
+}
+
 static char *
 get_app_profile_directory_name (const char *id)
 {
-  char *profile_dir;
-  char *encoded;
+  g_autofree char *gapplication_id = NULL;
 
-  profile_dir = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, NULL);
-  encoded = get_encoded_path (profile_dir);
-  g_free (profile_dir);
+  gapplication_id = get_gapplication_id_from_id (id);
+  if (!gapplication_id)
+    g_error ("Failed to get GApplication ID from app ID %s", id);
 
-  return encoded;
+  return get_encoded_path (gapplication_id);
 }
 
 static char *
 get_app_desktop_filename (const char *id)
 {
-  char *filename;
-  char *encoded;
+  g_autofree char *gapplication_id = NULL;
+  g_autofree char *filename = NULL;
 
-  filename = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, ".desktop", NULL);
-  encoded = get_encoded_path (filename);
-  g_free (filename);
+  /* Warning: the GApplication ID must exactly match the desktop file's
+   * basename. Don't overthink this or stuff will break, e.g. GNotification.
+   */
+  gapplication_id = get_gapplication_id_from_id (id);
+  if (!gapplication_id)
+    g_error ("Failed to get GApplication ID from app ID %s", id);
 
-  return encoded;
+  filename = g_strconcat (gapplication_id, ".desktop", NULL);
+  return get_encoded_path (filename);
 }
 
 const char *
-ephy_web_application_get_program_name_from_profile_directory (const char *profile_dir)
+ephy_web_application_get_gapplication_id_from_profile_directory (const char *profile_dir)
 {
   const char *name;
 
@@ -129,32 +203,21 @@ ephy_web_application_get_program_name_from_profile_directory (const char *profil
   if (g_str_has_prefix (name, "app-"))
     name += strlen ("app-");
 
-  if (!g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX)) {
-    g_warning ("Profile directory %s does not begin with required web app prefix %s", profile_dir, EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
+  if (!g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
+    g_warning ("Profile directory %s does not begin with required web app prefix %s", profile_dir, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
     return NULL;
   }
 
   return name;
 }
 
-static const char *
-get_app_id_from_program_name (const char *name)
-{
-  if (!g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX)) {
-    g_warning ("Program name %s does not begin with required prefix %s", name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
-    return NULL;
-  }
-
-  return name + strlen (EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
-}
-
 static const char *
 get_app_id_from_profile_directory (const char *profile_dir)
 {
-  const char *program_name;
+  const char *gapplication_id;
 
-  program_name = ephy_web_application_get_program_name_from_profile_directory (profile_dir);
-  return program_name ? get_app_id_from_program_name (program_name) : NULL;
+  gapplication_id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_dir);
+  return gapplication_id ? get_app_id_from_gapplication_id (gapplication_id) : NULL;
 }
 
 static char *
@@ -323,9 +386,12 @@ create_desktop_file (const char *id,
     g_free (path);
   }
 
-  wm_class = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, NULL);
+  wm_class = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, id, NULL);
   g_key_file_set_value (file, "Desktop Entry", "StartupWMClass", wm_class);
   g_free (wm_class);
+
+  g_key_file_set_value (file, "Desktop Entry", "X-Purism-FormFactor", "Workstation;Mobile;");
+
   data = g_key_file_to_data (file, NULL, NULL);
 
   desktop_file_path = g_build_filename (profile_dir, filename, NULL);
@@ -418,28 +484,65 @@ ephy_web_application_create (const char                *id,
 char *
 ephy_web_application_ensure_for_app_info (GAppInfo *app_info)
 {
-  char *id;
-  char *profile_dir;
+  g_autofree char *id = NULL;
+  g_autofree char *profile_dir = NULL;
+  g_autofree char *app_file = NULL;
+  int fd;
 
   id = ephy_web_application_get_app_id_from_name (g_app_info_get_name (app_info));
   profile_dir = ephy_web_application_get_profile_directory (id);
-  g_free (id);
 
+  /* Create the profile directory, populate it. */
   if (g_mkdir (profile_dir, 488) == -1) {
     if (errno == EEXIST)
-      return profile_dir;
+      return g_steal_pointer (&profile_dir);
 
-    g_free (profile_dir);
     return NULL;
   }
 
-  return profile_dir;
+  /* Skip migration for new web apps. */
+  ephy_profile_utils_set_migration_version_for_profile_dir (EPHY_PROFILE_MIGRATION_VERSION, profile_dir);
+
+  /* Create an .app file. */
+  app_file = g_build_filename (profile_dir, ".app", NULL);
+  fd = g_open (app_file, O_WRONLY | O_CREAT | O_TRUNC, 0644);
+  if (fd < 0) {
+    g_warning ("Failed to create .app file: %s", g_strerror (errno));
+    return NULL;
+  }
+  close (fd);
+
+  /* Create the deskop file. */
+  if (G_IS_DESKTOP_APP_INFO (app_info)) {
+    const char *source_name = NULL;
+    g_autofree char *dest_name = NULL;
+    g_autofree char *desktop_basename = NULL;
+    g_autoptr (GFile) source = NULL;
+    g_autoptr (GFile) dest = NULL;
+    g_autoptr (GError) error = NULL;
+
+    source_name = g_desktop_app_info_get_filename (G_DESKTOP_APP_INFO (app_info));
+    source = g_file_new_for_path (source_name);
+
+    desktop_basename = get_app_desktop_filename (id);
+    dest_name = g_build_filename (profile_dir, desktop_basename, NULL);
+    dest = g_file_new_for_path (dest_name);
+
+    g_file_copy (source, dest, G_FILE_COPY_NONE, NULL, NULL, NULL, &error);
+
+    if (error)
+      g_warning ("Couldn't copy desktop file: %s", error->message);
+
+    ephy_web_application_initialize_settings (profile_dir, EPHY_WEB_APPLICATION_SYSTEM);
+  }
+
+  return g_steal_pointer (&profile_dir);
 }
 
 void
 ephy_web_application_setup_from_profile_directory (const char *profile_directory)
 {
-  const char *program_name;
+  const char *gapplication_id;
   const char *id;
   char *app_icon;
   char *desktop_basename;
@@ -448,24 +551,23 @@ ephy_web_application_setup_from_profile_directory (const char *profile_directory
 
   g_assert (profile_directory != NULL);
 
-  program_name = ephy_web_application_get_program_name_from_profile_directory (profile_directory);
-  if (!program_name)
-    exit (1);
+  gapplication_id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_directory);
+  if (!gapplication_id)
+    g_error ("Failed to get GApplication ID from profile directory %s", profile_directory);
 
-  g_set_prgname (program_name);
+  /* FIXME: is this actually necessary? Probably not? */
+  g_set_prgname (gapplication_id);
 
-  id = get_app_id_from_program_name (program_name);
+  id = get_app_id_from_gapplication_id (gapplication_id);
   if (!id)
-    exit (1);
+    g_error ("Failed to get app ID from GApplication ID %s", gapplication_id);
 
   /* Get display name from desktop file */
   desktop_basename = get_app_desktop_filename (id);
   desktop_filename = g_build_filename (profile_directory, desktop_basename, NULL);
   desktop_info = g_desktop_app_info_new_from_filename (desktop_filename);
-  if (!desktop_info) {
-    g_warning ("Required desktop file not present at %s", desktop_filename);
-    exit (1);
-  }
+  if (!desktop_info)
+    g_error ("Required desktop file not present at %s", desktop_filename);
   g_set_application_name (g_app_info_get_name (G_APP_INFO (desktop_info)));
 
   app_icon = g_build_filename (profile_directory, EPHY_WEB_APP_ICON_NAME, NULL);
@@ -474,7 +576,7 @@ ephy_web_application_setup_from_profile_directory (const char *profile_directory
   /* We need to re-set this because we have already parsed the
    * options, which inits GTK+ and sets this as a side effect.
    */
-  gdk_set_program_class (program_name);
+  gdk_set_program_class (gapplication_id);
 
   g_free (app_icon);
   g_free (desktop_basename);
@@ -617,7 +719,7 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
 
     name = g_file_info_get_name (info);
     if ((only_legacy && g_str_has_prefix (name, "app-")) ||
-        (!only_legacy && g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX))) {
+        (!only_legacy && g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX))) {
       EphyWebApplication *app;
       char *profile_dir;
 
@@ -756,12 +858,17 @@ ephy_web_application_initialize_settings (const char                *profile_dir
   g_object_unref (settings);
   g_object_unref (web_app_settings);
 
-  if (options & EPHY_WEB_APPLICATION_MOBILE_CAPABLE) {
+  if (options) {
     path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", name, "webapp/", NULL);
     web_app_settings = g_settings_new_with_path (EPHY_PREFS_WEB_APP_SCHEMA, path);
     g_free (path);
 
-    g_settings_set_boolean (web_app_settings, EPHY_PREFS_WEB_APP_MOBILE_CAPABLE, TRUE);
+    if (options & EPHY_WEB_APPLICATION_MOBILE_CAPABLE)
+      g_settings_set_boolean (web_app_settings, EPHY_PREFS_WEB_APP_SHOW_NAVIGATION_BUTTONS, TRUE);
+
+    if (options & EPHY_WEB_APPLICATION_SYSTEM)
+      g_settings_set_boolean (web_app_settings, EPHY_PREFS_WEB_APP_SYSTEM, TRUE);
+
     g_object_unref (web_app_settings);
   }
 
@@ -921,3 +1028,20 @@ ephy_web_application_save (EphyWebApplication *app)
 
   return saved;
 }
+
+gboolean
+ephy_web_application_is_system (EphyWebApplication *app)
+{
+  GSettings *web_app_settings;
+  g_autofree char *profile_directory = NULL;
+  g_autofree char *name = NULL;
+  g_autofree char *path = NULL;
+
+  profile_directory = ephy_web_application_get_profile_directory (app->id);
+  name = g_path_get_basename (profile_directory);
+
+  path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", name, "webapp/", NULL);
+  web_app_settings = g_settings_new_with_path (EPHY_PREFS_WEB_APP_SCHEMA, path);
+
+  return g_settings_get_boolean (web_app_settings, EPHY_PREFS_WEB_APP_SYSTEM);
+}
