@@ -57,6 +57,7 @@ struct _EphyLocationEntry {
   GtkOverlay parent_instance;
 
   GtkWidget *url_entry;
+  GtkWidget *page_action_box;
   GtkWidget *bookmark;
   GtkWidget *bookmark_event_box;
   GtkWidget *reader_mode;
@@ -68,6 +69,7 @@ struct _EphyLocationEntry {
   GtkCssProvider *css_provider;
 
   gboolean reader_mode_active;
+  gboolean button_release_is_blocked;
 
   char *saved_text;
   char *jump_tab;
@@ -114,6 +116,44 @@ static void schedule_dns_prefetch (EphyLocationEntry *entry,
 G_DEFINE_TYPE_WITH_CODE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_OVERLAY,
                          G_IMPLEMENT_INTERFACE (EPHY_TYPE_TITLE_WIDGET,
                                                 ephy_location_entry_title_widget_interface_init))
+static gboolean
+entry_button_release (GtkWidget *widget,
+                      GdkEvent  *event,
+                      gpointer   user_data)
+{
+  EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (user_data);
+
+  if (((GdkEventButton *)event)->button != GDK_BUTTON_PRIMARY)
+    return GDK_EVENT_PROPAGATE;
+
+  gtk_editable_select_region (GTK_EDITABLE (entry->url_entry), 0, -1);
+
+  g_signal_handlers_block_by_func (widget, G_CALLBACK (entry_button_release), entry);
+  entry->button_release_is_blocked = TRUE;
+
+  return GDK_EVENT_STOP;
+}
+
+static gboolean
+entry_focus_out_event (GtkWidget *widget,
+                       GdkEvent  *event,
+                       gpointer   user_data)
+{
+  EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (user_data);
+
+  if (((GdkEventButton *)event)->button != GDK_BUTTON_PRIMARY)
+    return GDK_EVENT_PROPAGATE;
+
+  /* Unselect. */
+  gtk_editable_select_region (GTK_EDITABLE (entry->url_entry), 0, 0);
+
+  if (entry->button_release_is_blocked) {
+    g_signal_handlers_unblock_by_func (widget, G_CALLBACK (entry_button_release), entry);
+    entry->button_release_is_blocked = FALSE;
+  }
+
+  return GDK_EVENT_PROPAGATE;
+}
 
 static void
 editable_changed_cb (GtkEditable       *editable,
@@ -994,10 +1034,14 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_container_add (GTK_CONTAINER (event), box);
   g_signal_connect (G_OBJECT (box), "size-allocate", G_CALLBACK (button_box_size_allocated_cb), entry);
-  gtk_widget_set_halign (box, GTK_ALIGN_END);
   gtk_widget_set_valign (box, GTK_ALIGN_CENTER);
+  gtk_widget_set_halign (box, GTK_ALIGN_END);
   gtk_widget_show (box);
 
+  /* Page action box */
+  entry->page_action_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_pack_start (GTK_BOX (box), entry->page_action_box, FALSE, FALSE, 0);
+
   context = gtk_widget_get_style_context (box);
   gtk_style_context_add_class (context, "entry_icon_box");
 
@@ -1045,6 +1089,8 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   g_signal_connect (entry->url_entry, "suggestion-activated",
                     G_CALLBACK (ephy_location_entry_suggestion_activated), entry);
 
+  g_signal_connect (entry->url_entry, "button-release-event", G_CALLBACK (entry_button_release), entry);
+  g_signal_connect (entry->url_entry, "focus-out-event", G_CALLBACK (entry_focus_out_event), entry);
 
   controller = dzl_shortcut_controller_find (entry->url_entry);
   dzl_shortcut_controller_add_command_callback (controller,
@@ -1071,6 +1117,7 @@ ephy_location_entry_init (EphyLocationEntry *le)
 
   le->user_changed = FALSE;
   le->block_update = FALSE;
+  le->button_release_is_blocked = FALSE;
   le->saved_text = NULL;
 
   ephy_location_entry_construct_contents (le);
@@ -1274,14 +1321,7 @@ ephy_location_entry_reset (EphyLocationEntry *entry)
 void
 ephy_location_entry_focus (EphyLocationEntry *entry)
 {
-  GtkWidget *toplevel, *widget = GTK_WIDGET (entry->url_entry);
-
-  toplevel = gtk_widget_get_toplevel (widget);
-
-  gtk_editable_select_region (GTK_EDITABLE (entry->url_entry),
-                              0, -1);
-  gtk_window_set_focus (GTK_WINDOW (toplevel),
-                        widget);
+  gtk_widget_grab_focus (GTK_WIDGET (entry->url_entry));
 }
 
 void
@@ -1470,3 +1510,29 @@ ephy_location_entry_set_mobile_popdown (EphyLocationEntry *entry,
   else
     dzl_suggestion_entry_set_position_func (DZL_SUGGESTION_ENTRY (entry->url_entry), position_func, NULL, NULL);
 }
+
+void
+ephy_location_entry_page_action_add (EphyLocationEntry *entry,
+                                     GtkWidget         *action)
+{
+  gtk_box_pack_end (GTK_BOX (entry->page_action_box), action, FALSE, FALSE, 0);
+
+  gtk_widget_show (entry->page_action_box);
+}
+
+static void
+clear_page_actions (GtkWidget *child,
+                    gpointer   user_data)
+{
+  EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (user_data);
+
+  gtk_container_remove (GTK_CONTAINER (entry->page_action_box), child);
+}
+
+void
+ephy_location_entry_page_action_clear (EphyLocationEntry *entry)
+{
+  gtk_container_foreach (GTK_CONTAINER (entry->page_action_box), clear_page_actions, entry);
+
+  gtk_widget_hide (entry->page_action_box);
+}
