@@ -24,9 +24,11 @@
 #include "ephy-shell.h"
 
 #include "ephy-debug.h"
+#include "ephy-desktop-utils.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
+#include "ephy-firefox-sync-dialog.h"
 #include "ephy-gui.h"
 #include "ephy-header-bar.h"
 #include "ephy-history-dialog.h"
@@ -59,8 +61,10 @@ struct _EphyShell {
   EphyBookmarksManager *bookmarks_manager;
   EphyHistoryManager *history_manager;
   EphyOpenTabsManager *open_tabs_manager;
+  EphyWebExtensionManager *web_extension_manager;
   GNetworkMonitor *network_monitor;
   GtkWidget *history_dialog;
+  GtkWidget *firefox_sync_dialog;
   GObject *prefs_dialog;
   EphyShellStartupContext *local_startup_context;
   EphyShellStartupContext *remote_startup_context;
@@ -240,6 +244,18 @@ show_history (GSimpleAction *action,
   window_cmd_show_history (NULL, NULL, EPHY_WINDOW (window));
 }
 
+static void
+show_firefox_sync (GSimpleAction *action,
+                   GVariant      *parameter,
+                   gpointer       user_data)
+{
+  GtkWindow *window;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_show_firefox_sync (NULL, NULL, EPHY_WINDOW (window));
+}
+
 static void
 show_preferences (GSimpleAction *action,
                   GVariant      *parameter,
@@ -341,18 +357,6 @@ notification_clicked (GSimpleAction *action,
     webkit_notification_clicked (notification);
 }
 
-static void
-run_in_background_change_state (GSimpleAction *action,
-                                GVariant      *value,
-                                gpointer       user_data)
-{
-  gboolean active;
-
-  active = g_settings_get_boolean (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_RUN_IN_BACKGROUND);
-  g_simple_action_set_state (action, g_variant_new_boolean (!active));
-  g_settings_set_boolean (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_RUN_IN_BACKGROUND, !active);
-}
-
 static GActionEntry app_entries[] = {
   { "new-window", new_window, NULL, NULL, NULL },
   { "new-incognito", new_incognito_window, NULL, NULL, NULL },
@@ -360,6 +364,7 @@ static GActionEntry app_entries[] = {
   { "export-bookmarks", export_bookmarks, NULL, NULL, NULL },
   { "import-passwords", import_passwords, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
+  { "firefox-sync-dialog", show_firefox_sync, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "shortcuts", show_shortcuts, NULL, NULL, NULL },
   { "help", show_help, NULL, NULL, NULL },
@@ -379,7 +384,7 @@ static GActionEntry app_mode_app_entries[] = {
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
-  { "run-in-background", NULL, "b", "false", run_in_background_change_state},
+  { "run-in-background", NULL, NULL, "false", NULL},
   { "notification-clicked", notification_clicked, "t", NULL, NULL},
 };
 
@@ -468,6 +473,16 @@ run_in_background_get_mapping (GValue   *value,
   return TRUE;
 }
 
+static GVariant *
+run_in_background_set_mapping (const GValue       *value,
+                               const GVariantType *expected_type,
+                               gpointer            user_data)
+{
+  GVariant *var = g_value_get_variant (value);
+
+  return g_variant_new_boolean (g_variant_get_boolean (var));
+}
+
 static void
 ephy_shell_startup (GApplication *application)
 {
@@ -480,6 +495,15 @@ ephy_shell_startup (GApplication *application)
 
   hdy_init ();
 
+  /* If we are under Pantheon set the icon-theme and cursor-theme accordingly. */
+  if (is_desktop_pantheon ()) {
+    GtkSettings *settings = gtk_settings_get_default ();
+    g_object_set (settings,
+                  "gtk-icon-theme-name", "elementary",
+                  "gtk-cursor-theme-name", "elementary",
+                  NULL);
+  }
+
   /* We're not remoting; start our services */
 
   mode = ephy_embed_shell_get_mode (embed_shell);
@@ -522,11 +546,11 @@ ephy_shell_startup (GApplication *application)
     action = g_action_map_lookup_action (G_ACTION_MAP (application), "run-in-background");
     g_settings_bind_with_mapping (EPHY_SETTINGS_WEB_APP,
                                   EPHY_PREFS_WEB_APP_RUN_IN_BACKGROUND,
-                                  G_SIMPLE_ACTION (action),
+                                  action,
                                   "state",
-                                  G_SETTINGS_BIND_GET | G_SETTINGS_BIND_GET_NO_CHANGES,
+                                  G_SETTINGS_BIND_DEFAULT,
                                   run_in_background_get_mapping,
-                                  NULL,
+                                  run_in_background_set_mapping,
                                   NULL,
                                   NULL);
   }
@@ -593,7 +617,7 @@ portal_check (EphyShell *shell)
   if (g_network_monitor_get_connectivity (ephy_shell_get_net_monitor (shell)) == G_NETWORK_CONNECTIVITY_PORTAL) {
     EphyWindow *window = EPHY_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (shell)));
 
-    ephy_window_load_url (window, "http://nmcheck.gnome.org/");
+    ephy_link_open (EPHY_LINK (window), "http://nmcheck.gnome.org/", NULL, EPHY_LINK_NEW_TAB | EPHY_LINK_JUMP_TO);
   }
 }
 
@@ -611,8 +635,10 @@ ephy_shell_activate (GApplication *application)
   EphyShell *shell = EPHY_SHELL (application);
   EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (shell);
 
-  g_signal_connect (ephy_shell_get_net_monitor (shell), "notify::connectivity", G_CALLBACK (connectivity_changed), shell);
-  portal_check (shell);
+  if (!is_desktop_gnome () && !is_desktop_pantheon ()) {
+    g_signal_connect (ephy_shell_get_net_monitor (shell), "notify::connectivity", G_CALLBACK (connectivity_changed), shell);
+    portal_check (shell);
+  }
 
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_AUTOMATION) {
     WebKitWebContext *web_context = ephy_embed_shell_get_web_context (embed_shell);
@@ -909,23 +935,22 @@ webkit_notification_clicked_cb (WebKitNotification *notification,
 
   for (guint win_idx = 0; win_idx < g_list_length (windows); win_idx++) {
     EphyWindow *window;
-    GtkWidget *notebook;
+    EphyTabView *tab_view;
     int n_pages;
 
     window = EPHY_WINDOW (g_list_nth_data (windows, win_idx));
-
-    notebook = ephy_window_get_notebook (window);
-    n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+    tab_view = ephy_window_get_tab_view (window);
+    n_pages = ephy_tab_view_get_n_pages (tab_view);
 
     for (int i = 0; i < n_pages; i++) {
       EphyEmbed *embed;
       WebKitWebView *webview;
 
-      embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+      embed = EPHY_EMBED (ephy_tab_view_get_nth_page (tab_view, i));
       webview = WEBKIT_WEB_VIEW (ephy_embed_get_web_view (embed));
 
       if (webview == notification_webview) {
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), i);
+        ephy_tab_view_select_page (tab_view, GTK_WIDGET (embed));
         gtk_window_present (GTK_WINDOW (window));
         return;
       }
@@ -999,6 +1024,7 @@ ephy_shell_new_tab_full (EphyShell       *shell,
   EphyEmbed *embed = NULL;
   gboolean jump_to = FALSE;
   int position = -1;
+  EphyEmbed *parent = NULL;
 
   g_assert (EPHY_IS_SHELL (shell));
   g_assert (EPHY_IS_WINDOW (window));
@@ -1014,7 +1040,7 @@ ephy_shell_new_tab_full (EphyShell       *shell,
 
   if (flags & EPHY_NEW_TAB_APPEND_AFTER) {
     if (previous_embed)
-      position = ephy_window_get_position_for_new_embed (window, previous_embed);
+      parent = previous_embed;
     else
       g_warning ("Requested to append new tab after parent, but 'previous_embed' was NULL");
   }
@@ -1035,7 +1061,7 @@ ephy_shell_new_tab_full (EphyShell       *shell,
                                     "progress-bar-enabled", ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION,
                                     NULL));
   gtk_widget_show (GTK_WIDGET (embed));
-  ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, position, jump_to);
+  ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, parent, position, jump_to);
 
   if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 &&
       ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_TEST) {
@@ -1215,6 +1241,25 @@ ephy_shell_get_history_dialog (EphyShell *shell)
   return shell->history_dialog;
 }
 
+/**
+ * ephy_shell_get_firefox_sync_dialog:
+ *
+ * Return value: (transfer none):
+ **/
+GtkWidget *
+ephy_shell_get_firefox_sync_dialog (EphyShell *shell)
+{
+  if (shell->firefox_sync_dialog == NULL) {
+    shell->firefox_sync_dialog = ephy_firefox_sync_dialog_new ();
+    g_signal_connect (shell->firefox_sync_dialog,
+                      "destroy",
+                      G_CALLBACK (gtk_widget_destroyed),
+                      &shell->firefox_sync_dialog);
+  }
+
+  return shell->firefox_sync_dialog;
+}
+
 /**
  * ephy_shell_get_prefs_dialog:
  *
@@ -1244,9 +1289,10 @@ _ephy_shell_create_instance (EphyEmbedShellMode mode)
 
   if (mode == EPHY_EMBED_SHELL_MODE_APPLICATION) {
     const char *profile_dir = ephy_profile_dir ();
-    const char *web_id = ephy_web_application_get_program_name_from_profile_directory (profile_dir);
 
-    id = web_id;
+    id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_dir);
+    if (id == NULL)
+      g_error ("Cannot start web app instance: %s is not a valid profile directory", profile_dir);
   } else {
     id = APPLICATION_ID;
   }
@@ -1491,3 +1537,59 @@ ephy_shell_startup_finished (EphyShell *shell)
 {
   return shell->startup_finished;
 }
+
+EphyWebExtensionManager *
+ephy_shell_get_web_extension_manager (EphyShell *shell)
+{
+  g_assert (EPHY_IS_SHELL (shell));
+
+  if (shell->web_extension_manager == NULL)
+    shell->web_extension_manager = ephy_web_extension_manager_new ();
+
+  return shell->web_extension_manager;
+}
+
+
+/* Helper functions: better place for this? */
+EphyWebView *
+ephy_shell_get_web_view (EphyShell *shell,
+                         guint64    id)
+{
+  GList *windows;
+  GtkWindow *window;
+  EphyTabView *tab_view;
+
+  windows = gtk_application_get_windows (GTK_APPLICATION (shell));
+
+  for (GList *list = windows; list && list->data; list = list->next) {
+    window = GTK_WINDOW (list->data);
+    tab_view = ephy_window_get_tab_view (EPHY_WINDOW (window));
+
+    for (int i = 0; i < ephy_tab_view_get_n_pages (tab_view); i++) {
+      GtkWidget *page = ephy_tab_view_get_nth_page (tab_view, i);
+      EphyWebView *web_view = ephy_embed_get_web_view (EPHY_EMBED (page));
+
+      if (ephy_web_view_get_uid (web_view) == id)
+        return web_view;
+    }
+  }
+
+  return NULL;
+}
+
+EphyWebView *
+ephy_shell_get_active_web_view (EphyShell *shell)
+{
+  GtkWindow *window;
+  EphyTabView *tab_view;
+  GtkWidget *page;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (shell));
+  if (!window)
+    return NULL;
+
+  tab_view = ephy_window_get_tab_view (EPHY_WINDOW (window));
+  page = ephy_tab_view_get_selected_page (tab_view);
+
+  return ephy_embed_get_web_view (EPHY_EMBED (page));
+}
