@@ -20,8 +20,11 @@
 #include "ephy-suggestion-model.h"
 
 #include "ephy-embed-shell.h"
+#include "ephy-prefs.h"
 #include "ephy-search-engine-manager.h"
+#include "ephy-settings.h"
 #include "ephy-suggestion.h"
+#include "ephy-user-agent.h"
 #include "ephy-window.h"
 
 #include <dazzle.h>
@@ -37,6 +40,7 @@ struct _EphySuggestionModel {
   GSequence *items;
   GCancellable *icon_cancellable;
   guint num_custom_entries;
+  SoupSession *session;
 };
 
 enum {
@@ -62,6 +66,7 @@ ephy_suggestion_model_finalize (GObject *object)
   g_clear_object (&self->history_service);
   g_clear_pointer (&self->urls, g_sequence_free);
   g_clear_pointer (&self->items, g_sequence_free);
+  g_clear_object (&self->session);
 
   g_cancellable_cancel (self->icon_cancellable);
   g_clear_object (&self->icon_cancellable);
@@ -139,6 +144,7 @@ static void
 ephy_suggestion_model_init (EphySuggestionModel *self)
 {
   self->items = g_sequence_new (g_object_unref);
+  self->session = soup_session_new_with_options ("user-agent", ephy_user_agent_get (), NULL);
 }
 
 static GType
@@ -194,13 +200,44 @@ static gboolean
 should_add_bookmark_to_model (EphySuggestionModel *self,
                               const char          *search_string,
                               const char          *title,
-                              const char          *location)
+                              const char          *location,
+                              GSequence           *tags)
 {
   g_autofree gchar *search_casefold = g_utf8_casefold (search_string, -1);
   g_autofree gchar *title_casefold = g_utf8_casefold (title, -1);
   g_autofree gchar *location_casefold = g_utf8_casefold (location, -1);
+  g_autofree char *tag_string = NULL;
+  g_autofree char *tag_string_casefold = NULL;
+  char **tag_array = NULL;
+  g_auto (GStrv) search_terms = NULL;
+  GSequenceIter *tag_iter;
+  guint i;
+  gboolean ret = TRUE;
 
-  return strstr (title_casefold, search_casefold) || strstr (location_casefold, search_casefold);
+  tag_array = g_malloc0 ((g_sequence_get_length (tags) + 1) * sizeof (char *));
+
+  for (i = 0, tag_iter = g_sequence_get_begin_iter (tags);
+       !g_sequence_iter_is_end (tag_iter);
+       i++, tag_iter = g_sequence_iter_next (tag_iter)) {
+    tag_array[i] = g_sequence_get (tag_iter);
+  }
+
+  tag_string = g_strjoinv (" ", tag_array);
+  tag_string_casefold = g_utf8_casefold (tag_string, -1);
+  search_terms = g_strsplit (search_casefold, " ", -1);
+
+  for (i = 0; i < g_strv_length (search_terms); i++) {
+    if (!strstr (title_casefold, search_terms[i]) &&
+        !strstr (location_casefold, search_terms[i]) &&
+        (tag_string_casefold && !strstr (tag_string_casefold, search_terms[i]))) {
+      ret = FALSE;
+      break;
+    }
+  }
+
+  g_free (tag_array);
+
+  return ret;
 }
 
 static void
@@ -252,8 +289,12 @@ static gboolean
 append_suggestion (EphySuggestionModel *self,
                    EphySuggestion      *suggestion)
 {
+  if (g_sequence_lookup (self->urls, (char *)ephy_suggestion_get_uri (suggestion), (GCompareDataFunc)g_strcmp0, NULL))
+    return FALSE;
+
   if (self->num_custom_entries < MAX_URL_ENTRIES) {
-    g_sequence_append (self->items, suggestion);
+    g_sequence_append (self->items, g_object_ref (suggestion));
+    load_favicon (self, suggestion, ephy_suggestion_get_uri (suggestion));
     self->num_custom_entries++;
 
     return TRUE;
@@ -276,21 +317,21 @@ add_bookmarks (EphySuggestionModel *self,
        !g_sequence_iter_is_end (iter);
        iter = g_sequence_iter_next (iter)) {
     EphyBookmark *bookmark;
+    GSequence *tags;
     const char *url, *title;
 
     bookmark = g_sequence_get (iter);
 
     url = ephy_bookmark_get_url (bookmark);
-    title = ephy_bookmark_get_title (bookmark);
-
-    if (g_sequence_lookup (self->urls, (gpointer)url, (GCompareDataFunc)g_strcmp0,
-                           NULL))
+    if (g_sequence_lookup (self->urls, (gpointer)url, (GCompareDataFunc)g_strcmp0, NULL))
       continue;
 
+    title = ephy_bookmark_get_title (bookmark);
     if (strlen (title) == 0)
       title = url;
 
-    if (should_add_bookmark_to_model (self, query, title, url)) {
+    tags = ephy_bookmark_get_tags (bookmark);
+    if (should_add_bookmark_to_model (self, query, title, url, tags)) {
       EphySuggestion *suggestion;
       g_autofree gchar *escaped_title = NULL;
       g_autofree gchar *markup = NULL;
@@ -317,42 +358,6 @@ add_bookmarks (EphySuggestionModel *self,
   return added;
 }
 
-static guint
-add_history (EphySuggestionModel *self,
-             GList               *urls,
-             const char          *query)
-{
-  guint added = 0;
-
-  for (const GList *p = urls; p != NULL; p = p->next) {
-    EphyHistoryURL *url = (EphyHistoryURL *)p->data;
-    EphySuggestion *suggestion;
-    g_autofree gchar *escaped_title = NULL;
-    g_autofree gchar *markup = NULL;
-    const gchar *title = url->title;
-
-    if (g_sequence_lookup (self->urls, url->url, (GCompareDataFunc)g_strcmp0,
-                           NULL))
-      continue;
-
-    if (strlen (url->title) == 0)
-      title = url->url;
-
-    escaped_title = g_markup_escape_text (title, -1);
-
-    markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-    suggestion = ephy_suggestion_new (markup, title, url->url);
-    load_favicon (self, suggestion, url->url);
-
-    if (append_suggestion (self, suggestion))
-      added++;
-    else
-      break;
-  }
-
-  return added;
-}
-
 static guint
 add_search_engines (EphySuggestionModel *self,
                     const char          *query)
@@ -402,10 +407,9 @@ add_tabs (EphySuggestionModel *self,
   GApplication *application;
   EphyEmbedShell *shell;
   EphyWindow *window;
-  GtkWidget *notebook;
+  EphyTabView *tab_view;
   GList *windows;
-  gint n_pages;
-  gint current;
+  gint n_pages, selected;
   guint added = 0;
 
   shell = ephy_embed_shell_get_default ();
@@ -415,9 +419,9 @@ add_tabs (EphySuggestionModel *self,
   for (guint win_idx = 0; win_idx < g_list_length (windows); win_idx++) {
     window = EPHY_WINDOW (g_list_nth_data (windows, win_idx));
 
-    notebook = ephy_window_get_notebook (window);
-    n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
-    current = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+    tab_view = ephy_window_get_tab_view (window);
+    n_pages = ephy_tab_view_get_n_pages (tab_view);
+    selected = ephy_tab_view_get_selected_index (tab_view);
 
     for (int i = 0; i < n_pages; i++) {
       EphyEmbed *embed;
@@ -433,10 +437,10 @@ add_tabs (EphySuggestionModel *self,
       g_autofree gchar *display_address_casefold = NULL;
       g_autofree gchar *query_casefold = NULL;
 
-      if (win_idx == 0 && i == current)
+      if (win_idx == 0 && i == selected)
         continue;
 
-      embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+      embed = EPHY_EMBED (ephy_tab_view_get_nth_page (tab_view, i));
       webview = ephy_embed_get_web_view (embed);
       display_address = ephy_web_view_get_display_address (webview);
       url = ephy_web_view_get_address (webview);
@@ -471,22 +475,54 @@ add_tabs (EphySuggestionModel *self,
   return added;
 }
 
-static void
-query_completed_cb (EphyHistoryService *service,
-                    gboolean            success,
-                    gpointer            result_data,
-                    gpointer            user_data)
+typedef struct {
+  char *query;
+  gboolean include_search_engines;
+  GSequence *history;
+  GSequence *google_suggestions;
+  int active_sources;
+} QueryData;
+
+static QueryData *
+query_data_new (const char *query,
+                gboolean    include_search_engines)
 {
-  GTask *task = user_data;
-  EphySuggestionModel *self;
-  const gchar *query;
-  GList *urls;
+  QueryData *data;
+
+  data = g_malloc0 (sizeof (QueryData));
+  data->query = g_strdup (query);
+  data->include_search_engines = include_search_engines;
+  data->history = g_sequence_new (g_object_unref);
+  data->google_suggestions = g_sequence_new (g_object_unref);
+
+  return data;
+}
+
+static void
+query_data_free (QueryData *data)
+{
+  g_assert (data != NULL);
+  g_clear_pointer (&data->history, g_sequence_free);
+  g_clear_pointer (&data->google_suggestions, g_sequence_free);
+  g_free (data->query);
+  g_free (data);
+}
+
+static void
+query_collection_done (EphySuggestionModel *self,
+                       GTask               *task)
+{
+  QueryData *data;
   guint removed;
   guint added = 0;
 
   self = g_task_get_source_object (task);
-  query = g_task_get_task_data (task);
-  urls = (GList *)result_data;
+  data = g_task_get_task_data (task);
+
+  if (--data->active_sources) {
+    g_object_unref (task);
+    return;
+  }
 
   g_cancellable_cancel (self->icon_cancellable);
   g_clear_object (&self->icon_cancellable);
@@ -501,11 +537,31 @@ query_completed_cb (EphyHistoryService *service,
   self->items = g_sequence_new (g_object_unref);
   self->num_custom_entries = 0;
 
-  if (strlen (query) > 0) {
-    added = add_tabs (self, query);
-    added += add_bookmarks (self, query);
-    added += add_history (self, urls, query);
-    added += add_search_engines (self, query);
+  if (strlen (data->query) > 0) {
+    added = add_tabs (self, data->query);
+
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->google_suggestions); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      if (append_suggestion (self, tmp))
+        added++;
+      else
+        break;
+    }
+
+    added += add_bookmarks (self, data->query);
+
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->history); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      if (append_suggestion (self, tmp))
+        added++;
+      else
+        break;
+    }
+
+    if (data->include_search_engines)
+      added += add_search_engines (self, data->query);
   }
 
   g_list_model_items_changed (G_LIST_MODEL (self), 0, removed, added);
@@ -514,9 +570,123 @@ query_completed_cb (EphyHistoryService *service,
   g_object_unref (task);
 }
 
+static void
+history_query_completed_cb (EphyHistoryService *service,
+                            gboolean            success,
+                            gpointer            result_data,
+                            gpointer            user_data)
+{
+  GTask *task = user_data;
+  EphySuggestionModel *self;
+  QueryData *data;
+  GList *urls;
+
+  self = g_task_get_source_object (task);
+  data = g_task_get_task_data (task);
+  urls = (GList *)result_data;
+
+  if (strlen (data->query) > 0) {
+    for (const GList *p = urls; p != NULL; p = p->next) {
+      EphyHistoryURL *url = (EphyHistoryURL *)p->data;
+      EphySuggestion *suggestion;
+      g_autofree gchar *escaped_title = NULL;
+      g_autofree gchar *markup = NULL;
+      const gchar *title = url->title;
+
+      if (strlen (url->title) == 0)
+        title = url->url;
+
+      escaped_title = g_markup_escape_text (title, -1);
+
+      markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
+      suggestion = ephy_suggestion_new (markup, title, url->url);
+
+      g_sequence_append (data->history, g_steal_pointer (&suggestion));
+    }
+  }
+
+  query_collection_done (self, g_steal_pointer (&task));
+}
+
+static void
+google_search_suggestions_cb (SoupSession *session,
+                              SoupMessage *msg,
+                              gpointer     user_data)
+{
+  GTask *task = G_TASK (user_data);
+  EphySuggestionModel *self = g_task_get_source_object (task);
+  EphyEmbedShell *shell;
+  EphySearchEngineManager *manager;
+  QueryData *data;
+  JsonNode *node;
+  JsonArray *array;
+  JsonArray *suggestions;
+  char *engine;
+  int added = 0;
+
+  if (msg->status_code != 200)
+    goto out;
+
+  shell = ephy_embed_shell_get_default ();
+  manager = ephy_embed_shell_get_search_engine_manager (shell);
+  engine = ephy_search_engine_manager_get_default_engine (manager);
+
+  node = json_from_string (msg->response_body->data, NULL);
+  if (!node || !JSON_NODE_HOLDS_ARRAY (node)) {
+    g_warning ("Google search suggestion response is not a valid JSON object: %s", msg->response_body->data);
+    goto out;
+  }
+
+  array = json_node_get_array (node);
+  suggestions = json_array_get_array_element (array, 1);
+  data = g_task_get_task_data (task);
+
+  for (guint i = 0; i < json_array_get_length (suggestions) && added < 5; i++) {
+    EphySuggestion *suggestion;
+    const char *str = json_array_get_string_element (suggestions, i);
+    g_autofree char *address = NULL;
+    g_autofree char *escaped_title = NULL;
+    g_autofree char *markup = NULL;
+
+    address = ephy_search_engine_manager_build_search_address (manager, engine, str);
+    escaped_title = g_markup_escape_text (str, -1);
+    markup = dzl_fuzzy_highlight (escaped_title, str, FALSE);
+    suggestion = ephy_suggestion_new (markup, engine, address);
+
+    g_sequence_append (data->google_suggestions, g_steal_pointer (&suggestion));
+    added++;
+  }
+
+out:
+  query_collection_done (self, g_steal_pointer (&task));
+}
+
+static void
+google_search_suggestions_query (EphySuggestionModel *self,
+                                 const gchar         *query,
+                                 GTask               *task)
+{
+  SoupMessage *msg;
+  g_auto (GStrv) split = g_strsplit (query, " ", -1);
+  g_autofree char *url = NULL;
+  g_autofree char *escaped_query = NULL;
+
+  if (g_strv_length (split) < 2) {
+    query_collection_done (self, g_steal_pointer (&task));
+    return;
+  }
+
+  escaped_query = g_markup_escape_text (query, -1);
+  url = g_strdup_printf ("http://suggestqueries.google.com/complete/search?client=firefox&q=%s", escaped_query);
+  msg = soup_message_new (SOUP_METHOD_GET, url);
+
+  soup_session_queue_message (self->session, g_steal_pointer (&msg), google_search_suggestions_cb, g_steal_pointer (&task));
+}
+
 void
 ephy_suggestion_model_query_async (EphySuggestionModel *self,
                                    const gchar         *query,
+                                   gboolean             include_search_engines,
                                    GCancellable        *cancellable,
                                    GAsyncReadyCallback  callback,
                                    gpointer             user_data)
@@ -524,6 +694,7 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
   GTask *task = NULL;
   char **strings;
   GList *qlist = NULL;
+  QueryData *data;
 
   g_assert (EPHY_IS_SUGGESTION_MODEL (self));
   g_assert (query != NULL);
@@ -531,20 +702,31 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
 
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, ephy_suggestion_model_query_async);
-  g_task_set_task_data (task, g_strdup (query), g_free);
+
+  data = query_data_new (query, include_search_engines);
+  g_task_set_task_data (task, data, (GDestroyNotify)query_data_free);
 
   /* Split the search string. */
   strings = g_strsplit (query, " ", -1);
   for (guint i = 0; strings[i]; i++)
     qlist = g_list_append (qlist, g_strdup (strings[i]));
 
+  /* History */
+  data->active_sources = 1;
+
+  /* Google Search Suggestions */
+  if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_USE_GOOGLE_SEARCH_SUGGESTIONS)) {
+    data->active_sources++;
+    google_search_suggestions_query (self, query, g_object_ref (task));
+  }
+
   ephy_history_service_find_urls (self->history_service,
                                   0, 0,
                                   MAX_URL_ENTRIES, 0,
                                   qlist,
                                   EPHY_HISTORY_SORT_MOST_VISITED,
                                   cancellable,
-                                  (EphyHistoryJobCallback)query_completed_cb,
+                                  (EphyHistoryJobCallback)history_query_completed_cb,
                                   task);
 
   g_strfreev (strings);
